@@ -349,13 +349,18 @@ def bundle_icons(verbose=False, external=True, download_list=None):
         shutil.copyfile(icon, dest_dir / os.path.basename(icon))
 
 def patch_tabulator():
-    # https://github.com/olifolkerd/tabulator/issues/4421
-    path = BUNDLE_DIR / 'datatabulator' / 'tabulator-tables@6.2.1' / 'dist' / 'js' / 'tabulator.min.js'
+    path = BUNDLE_DIR / 'datatabulator' / 'tabulator-tables@6.3.0' / 'dist' / 'js' / 'tabulator.min.js'
     text = path.read_text()
+    # https://github.com/olifolkerd/tabulator/issues/4421
     old = '"focus"!==this.options("editTriggerEvent")&&"click"!==this.options("editTriggerEvent")'
     new = '"click"!==this.options("editTriggerEvent")'
     assert text.count(old) == 1
     text = text.replace(old, new)
+    # https://github.com/olifolkerd/tabulator/pull/4598
+    old = '(i=!0,this.subscribed("table-resize")?this.dispatch("table-resize"):this.redraw())'
+    new = '(i=!0,this.redrawing||(this.redrawing=!0,this.subscribed("table-resize")?this.dispatch("table-resize"):this.redraw(),this.redrawing=!1))'
+    assert text.count(old) == 1
+    text = text.replace(old, new)
     path.write_text(text)
 
 def bundle_resources(verbose=False, external=True):
