@@ -3,6 +3,8 @@ The config module supplies the global config object and the extension
 which provides convenient support for  loading and configuring panel
 components.
 """
+from __future__ import annotations
+
 import ast
 import copy
 import importlib
@@ -13,6 +15,7 @@ import warnings
 
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import contextmanager
+from typing import TYPE_CHECKING, Any, ClassVar
 from weakref import WeakKeyDictionary
 
 import param
@@ -24,7 +27,9 @@ from pyviz_comms import (
 from .__version import __version__
 from .io.logging import panel_log_handler
 from .io.state import state
-from .util import param_watchers
+
+if TYPE_CHECKING:
+    from bokeh.document import Document
 
 _LOCAL_DEV_VERSION = (
     any(v in __version__ for v in ('post', 'dirty'))
@@ -203,12 +208,12 @@ class _config(_base_config):
         information about user sessions. A value of -1 indicates an
         unlimited history.""")
 
-    sizing_mode = param.ObjectSelector(default=None, objects=[
+    sizing_mode = param.Selector(default=None, objects=[
         'fixed', 'stretch_width', 'stretch_height', 'stretch_both',
         'scale_width', 'scale_height', 'scale_both', None], doc="""
         Specify the default sizing mode behavior of panels.""")
 
-    template = param.ObjectSelector(default=None, doc="""
+    template = param.Selector(default=None, doc="""
         The default template to render served applications into.""")
 
     throttled = param.Boolean(default=False, doc="""
@@ -222,12 +227,12 @@ class _config(_base_config):
         default='DEBUG', objects=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
         doc="Log level of the Admin Panel logger")
 
-    _comms = param.ObjectSelector(
+    _comms = param.Selector(
         default='default', objects=['default', 'ipywidgets', 'vscode', 'colab'], doc="""
         Whether to render output in Jupyter with the default Jupyter
         extension or use the jupyter_bokeh ipywidget model.""")
 
-    _console_output = param.ObjectSelector(default='accumulate', allow_None=True,
+    _console_output = param.Selector(default='accumulate', allow_None=True,
                                  objects=['accumulate', 'replace', 'disable',
                                           False], doc="""
         How to log errors and stdout output triggered by callbacks
@@ -271,7 +276,7 @@ class _config(_base_config):
         or filepath containing JSON to use with the basic auth
         provider.""")
 
-    _oauth_provider = param.ObjectSelector(
+    _oauth_provider = param.Selector(
         default=None, allow_None=True, objects=[], doc="""
         Select between a list of authentication providers.""")
 
@@ -310,11 +315,11 @@ class _config(_base_config):
         Whether to inline JS and CSS resources. If disabled, resources
         are loaded from CDN if one is available.""")
 
-    _theme = param.ObjectSelector(default=None, objects=['default', 'dark'], allow_None=True, doc="""
+    _theme = param.Selector(default=None, objects=['default', 'dark'], allow_None=True, doc="""
         The theme to apply to components.""")
 
     # Global parameters that are shared across all sessions
-    _globals = {
+    _globals: ClassVar[set[str]] = {
         'admin_plugins', 'autoreload', 'comms', 'cookie_secret',
         'nthreads', 'oauth_provider', 'oauth_expiry', 'oauth_key',
         'oauth_secret', 'oauth_jwt_user', 'oauth_redirect_uri',
@@ -325,7 +330,7 @@ class _config(_base_config):
 
     _truthy = ['True', 'true', '1', True, 1]
 
-    _session_config = WeakKeyDictionary()
+    _session_config: ClassVar[WeakKeyDictionary[Document, dict[str, Any]]] = WeakKeyDictionary()
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -407,7 +412,7 @@ class _config(_base_config):
             if state.curdoc not in self._session_config:
                 self._session_config[state.curdoc] = {}
             self._session_config[state.curdoc][attr] = value
-            watchers = param_watchers(self).get(attr, {}).get('value', [])
+            watchers = self.param.watchers.get(attr, {}).get('value', [])
             for w in watchers:
                 w.fn()
         elif f'_{attr}' in _config._parameter_set and hasattr(self, f'_{attr}_'):
@@ -654,9 +659,9 @@ class panel_extension(_pyviz_extension):
     will be using the `FastListTemplate`.
     """
 
-    _loaded = False
+    _loaded: bool = False
 
-    _imports = {
+    _imports: ClassVar[dict[str, str]] = {
         'ace': 'panel.models.ace',
         'codeeditor': 'panel.models.ace',
         'deckgl': 'panel.models.deckgl',
@@ -679,7 +684,7 @@ class panel_extension(_pyviz_extension):
     # Check whether these are loaded before rendering (if any item
     # in the list is available the extension will be confidered as
     # loaded)
-    _globals = {
+    _globals: ClassVar[dict[str, list[str]]] = {
         'deckgl': ['deck'],
         'echarts': ['echarts'],
         'filedropper': ['FilePond'],
@@ -696,9 +701,9 @@ class panel_extension(_pyviz_extension):
         'vtk': ['vtk']
     }
 
-    _loaded_extensions = []
+    _loaded_extensions: list[str] = []
 
-    _comms_detected_before = False
+    _comms_detected_before: bool = False
 
     def __call__(self, *args, **params):
         from bokeh.core.has_props import _default_resolver
