@@ -7,6 +7,7 @@ import os
 import pathlib
 import uuid
 
+from collections.abc import Sequence
 from typing import IO, Any, Literal
 
 import bokeh
@@ -56,7 +57,7 @@ PYODIDE_JS = f'<script src="{PYODIDE_URL}"></script>'
 PYODIDE_PYC_JS = f'<script src="{PYODIDE_PYC_URL}"></script>'
 LOCAL_PREFIX = './'
 
-MINIMUM_VERSIONS = {}
+MINIMUM_VERSIONS: dict[str, str] = {}
 
 ICON_DIR = DIST_DIR / 'images'
 PWA_IMAGES = [
@@ -133,7 +134,7 @@ def make_index(files, title=None, manifest=True):
         favicon=favicon, title=title, PANEL_CDN=CDN_DIST
     )
 
-def build_pwa_manifest(files, title=None, **kwargs):
+def build_pwa_manifest(files, title=None, **kwargs) -> str:
     if len(files) > 1:
         title = title or 'Panel Applications'
         path = 'index.html'
@@ -148,7 +149,7 @@ def build_pwa_manifest(files, title=None, **kwargs):
 
 def script_to_html(
     filename: str | os.PathLike | IO,
-    requirements: Literal['auto'] | list[str] = 'auto',
+    requirements: list[str] | Literal['auto'] | os.PathLike = 'auto',
     js_resources: Literal['auto'] | list[str] = 'auto',
     css_resources: Literal['auto'] | list[str] | None = 'auto',
     runtime: Runtimes = 'pyodide',
@@ -159,7 +160,7 @@ def script_to_html(
     http_patch: bool = True,
     inline: bool = False,
     compiled: bool = True
-) -> str:
+) -> tuple[str, str | None]:
     """
     Converts a Panel or Bokeh script to a standalone WASM Python
     application.
@@ -168,7 +169,7 @@ def script_to_html(
     ---------
     filename: str | Path | IO
         The filename of the Panel/Bokeh application to convert.
-    requirements: 'auto' | List[str]
+    requirements: 'auto' | List[str] | os.PathLike
         The list of requirements to include (in addition to Panel).
     js_resources: 'auto' | List[str]
         The list of JS resources to include in the exported HTML.
@@ -200,7 +201,7 @@ def script_to_html(
         app_name = '.'.join(path.name.split('.')[:-1])
         app = build_single_handler_application(str(path.absolute()))
     document = Document()
-    document._session_context = lambda: MockSessionContext(document=document)
+    document._session_context = lambda: MockSessionContext(document=document)  # type: ignore
     with set_curdoc(document):
         app.initialize_document(document)
         state._on_load(None)
@@ -214,19 +215,23 @@ def script_to_html(
         )
 
     if requirements == 'auto':
-        requirements = find_requirements(source)
+        requirement_list = find_requirements(source)
     elif isinstance(requirements, str) and pathlib.Path(requirements).is_file():
-        requirements = pathlib.Path(requirements).read_text(encoding='utf-8').splitlines()
+        requirement_list = pathlib.Path(requirements).read_text(encoding='utf-8').splitlines()
         try:
             from packaging.requirements import Requirement
-            requirements = [
-                r2 for r in requirements
+            requirement_list = [
+                r2 for r in requirement_list
                 if (r2 := r.split("#")[0].strip()) and Requirement(r2)
             ]
         except Exception as e:
             raise ValueError(
                 f'Requirements parser raised following error: {e}'
             ) from e
+    elif isinstance(requirements, list):
+        requirement_list = requirements
+    else:
+        raise ValueError(f'Could not resolve requirements file {requirements}')
 
     # Environment
     if panel_version == 'local':
@@ -243,7 +248,7 @@ def script_to_html(
     if http_patch:
         base_reqs.append('pyodide-http==0.2.1')
     reqs = base_reqs + [
-        req for req in requirements if req not in ('panel', 'bokeh')
+        req for req in requirement_list if req not in ('panel', 'bokeh')
     ]
     for name, min_version in MINIMUM_VERSIONS.items():
         if any(name in req for req in reqs):
@@ -414,11 +419,9 @@ def convert_app(
 
     with open(dest_path / filename, 'w', encoding="utf-8") as out:
         out.write(html)
-    if runtime == 'pyscript-worker':
-        with open(dest_path / f'{name}.py', 'w', encoding="utf-8") as out:
-            out.write(worker)
-    elif runtime == 'pyodide-worker':
-        with open(dest_path / f'{name}.js', 'w', encoding="utf-8") as out:
+    if 'worker' in runtime and worker:
+        ext = 'py' if runtime.startswith('pyscript') else 'js'
+        with open(dest_path / f'{name}.{ext}', 'w', encoding="utf-8") as out:
             out.write(worker)
     if verbose:
         print(f'Successfully converted {app} to {runtime} target and wrote output to {filename}.')
@@ -426,8 +429,8 @@ def convert_app(
 
 
 def _convert_process_pool(
-    apps: list[str],
-    dest_path: str | None = None,
+    apps: Sequence[str | os.PathLike],
+    dest_path: os.PathLike | str | None = None,
     max_workers: int = 4,
     requirements: list[str] | Literal['auto'] | os.PathLike = 'auto',
     **kwargs
@@ -460,7 +463,7 @@ def _convert_process_pool(
     return files
 
 def convert_apps(
-    apps: str | os.PathLike | list[str | os.PathLike],
+    apps: str | os.PathLike | Sequence[str | os.PathLike],
     dest_path: str | os.PathLike | None = None,
     title: str | None = None,
     runtime: Runtimes = 'pyodide-worker',
@@ -543,18 +546,25 @@ def convert_apps(
         app_requirements = requirements
 
     kwargs = {
-        'requirements': app_requirements, 'runtime': runtime,
-        'prerender': prerender, 'manifest': manifest,
-        'panel_version': panel_version, 'http_patch': http_patch,
-        'inline': inline, 'verbose': verbose, 'compiled': compiled,
+        'runtime': runtime,
+        'prerender': prerender,
+        'manifest': manifest,
+        'panel_version': panel_version,
+        'http_patch': http_patch,
+        'inline': inline,
+        'verbose': verbose,
+        'compiled': compiled,
         'local_prefix': local_prefix
     }
 
     if state._is_pyodide:
-        files = dict(convert_app(app, dest_path, **kwargs) for app in apps)
+        files = {
+            app: convert_app(app, dest_path, requirements=app_requirements, **kwargs)  # type: ignore
+            for app in apps
+        }
     else:
         files = _convert_process_pool(
-            apps, dest_path, max_workers=max_workers, **kwargs
+            apps, dest_path, max_workers=max_workers, requirements=app_requirements, **kwargs
         )
 
     if build_index and len(files) >= 1:
