@@ -12,31 +12,29 @@ import threading
 import time
 import weakref
 
+from collections.abc import Callable, Iterator, Sequence
 from contextlib import contextmanager
 from functools import partial, wraps
-from typing import (
-    TYPE_CHECKING, Any, Callable, Iterator,
-)
+from typing import TYPE_CHECKING, Any
+from weakref import WeakKeyDictionary
 
 from bokeh.application.application import SessionContext
-from bokeh.core.serialization import Serializable
 from bokeh.document.document import Document
-from bokeh.document.events import (
-    ColumnDataChangedEvent, ColumnsPatchedEvent, ColumnsStreamedEvent,
-    DocumentChangedEvent, MessageSentEvent, ModelChangedEvent,
-)
+from bokeh.document.events import DocumentChangedEvent, DocumentPatchedEvent
 from bokeh.model.util import visit_immediate_value_references
 from bokeh.models import CustomJS
 
 from ..config import config
-from ..util import param_watchers
 from .loading import LOADING_INDICATOR_CSS_CLASS
 from .model import monkeypatch_events  # noqa: F401 API import
 from .state import curdoc_locked, state
 
 if TYPE_CHECKING:
+    from asyncio.futures import Future
+
     from bokeh.core.enums import HoldPolicyType
     from bokeh.core.has_props import HasProps
+    from bokeh.document import Document
     from bokeh.protocol.message import Message
     from bokeh.server.connection import ServerConnection
     from pyviz_comms import Comm
@@ -47,19 +45,15 @@ logger = logging.getLogger(__name__)
 # Private API
 #---------------------------------------------------------------------
 
-DISPATCH_EVENTS = (
-    ColumnDataChangedEvent, ColumnsPatchedEvent, ColumnsStreamedEvent,
-    ModelChangedEvent, MessageSentEvent
-)
 GC_DEBOUNCE = 5
-_WRITE_FUTURES = weakref.WeakKeyDictionary()
-_WRITE_MSGS = weakref.WeakKeyDictionary()
-_WRITE_BLOCK = weakref.WeakKeyDictionary()
+_WRITE_FUTURES: WeakKeyDictionary[Document, list[Future]] = WeakKeyDictionary()
+_WRITE_MSGS: WeakKeyDictionary[Document, dict[ServerConnection, list[Message]]] = WeakKeyDictionary()
+_WRITE_BLOCK: WeakKeyDictionary[Document, bool] = WeakKeyDictionary()
 
 _panel_last_cleanup = None
-_write_tasks = []
+_write_tasks: list[asyncio.Task] = []
 
-extra_socket_handlers = {}
+extra_socket_handlers: dict[type, Callable[[Any], None]] = {}
 
 @dataclasses.dataclass
 class Request:
@@ -120,10 +114,10 @@ def _cleanup_doc(doc, destroy=True):
                 pane._hooks = []
                 for p in pane.select():
                     p._hooks = []
-                    param_watchers(p, {})
+                    p.param.watchers = {}
                     p._documents = {}
                     p._internal_callbacks = {}
-            param_watchers(pane, {})
+            pane.param.watchers = {}
             pane._documents = {}
             pane._internal_callbacks = {}
         else:
@@ -281,11 +275,11 @@ def retrigger_events(doc: Document, events: list[DocumentChangedEvent]):
 def write_events(
     doc: Document,
     connections: list[ServerConnection],
-    events: list[DocumentChangedEvent]
+    events: list[DocumentPatchedEvent]
 ):
     from tornado.websocket import WebSocketHandler
 
-    futures = []
+    futures: list[Future] = []
     for conn in connections:
         if isinstance(conn._socket, WebSocketHandler):
             futures += dispatch_tornado(conn, events)
@@ -307,7 +301,7 @@ def write_events(
 def schedule_write_events(
     doc: Document,
     connections: list[ServerConnection],
-    events: list[DocumentChangedEvent]
+    events: list[DocumentPatchedEvent]
 ):
     # Set up write locks
     _WRITE_BLOCK[doc] = True
@@ -386,16 +380,19 @@ def with_lock(func: Callable) -> Callable:
 
 def dispatch_tornado(
     conn: ServerConnection,
-    events: list[DocumentChangedEvent] | None = None,
+    events: list[DocumentPatchedEvent] | None = None,
     msg: Message | None = None
-):
+) -> Sequence[Future]:
     from tornado.websocket import WebSocketHandler
     socket = conn._socket
     ws_conn = getattr(socket, 'ws_connection', False)
     if not ws_conn or ws_conn.is_closing(): # type: ignore
         return []
-    if msg is None:
-        msg = conn.protocol.create('PATCH-DOC', events)
+    elif msg is None:
+        if events:
+            msg = conn.protocol.create('PATCH-DOC', events)
+        else:
+            return []
     futures = [
         WebSocketHandler.write_message(socket, msg.header_json),
         WebSocketHandler.write_message(socket, msg.metadata_json),
@@ -412,12 +409,17 @@ def dispatch_tornado(
 
 def dispatch_django(
     conn: ServerConnection,
-    events: list[DocumentChangedEvent] | None = None,
+    events: list[DocumentPatchedEvent] | None = None,
     msg: Message | None = None
-):
+) -> Sequence[Future]:
     socket = conn._socket
     if msg is None:
-        msg = conn.protocol.create('PATCH-DOC', events)
+        return []
+    elif msg is None:
+        if events:
+            msg = conn.protocol.create('PATCH-DOC', events)
+        else:
+            return
     futures = [
         socket.send(text_data=msg.header_json),
         socket.send(text_data=msg.metadata_json),
@@ -489,7 +491,7 @@ def unlocked(policy: HoldPolicyType = 'combine') -> Iterator:
         curdoc.callbacks._held_events = []
         monkeypatch_events(events)
         for event in events:
-            if isinstance(event, DISPATCH_EVENTS) and not locked:
+            if isinstance(event, DocumentPatchedEvent) and not locked:
                 writeable_events.append(event)
             else:
                 remaining_events.append(event)
@@ -507,8 +509,8 @@ def unlocked(policy: HoldPolicyType = 'combine') -> Iterator:
             # the message reflects the event at the time it was generated
             # potentially avoiding issues serializing subsequent models
             # which assume the serializer has previously seen them.
-            serializable_events = [e for e in remaining_events if isinstance(e, Serializable)]
-            held_events = [e for e in remaining_events if not isinstance(e, Serializable)]
+            serializable_events = [e for e in remaining_events if isinstance(e, DocumentPatchedEvent)]
+            held_events = [e for e in remaining_events if not isinstance(e, DocumentPatchedEvent)]
             if serializable_events:
                 try:
                     schedule_write_events(curdoc, connections, serializable_events)
