@@ -16,7 +16,6 @@ from bokeh.core.property.bases import Property
 from bokeh.models import CustomJS
 from param.parameterized import Watcher
 
-from ..util import param_watchers
 from .model import add_to_doc, diff
 from .state import state
 
@@ -82,7 +81,7 @@ def save_dict(state, key=(), depth=0, max_depth=None, save_path='', load_path=No
 
 
 def get_watchers(reactive):
-    return [w for pwatchers in param_watchers(reactive).values()
+    return [w for pwatchers in reactive.param.watchers.values()
             for awatchers in pwatchers.values() for w in awatchers]
 
 
@@ -158,7 +157,7 @@ def links_to_jslinks(model, widget):
 
         mappings = []
         for pname, tgt_spec in link.links.items():
-            if Watcher(*link[:-4]) in param_watchers(widget)[pname]['value']:
+            if Watcher(*link[:-4]) in widget.param.watchers[pname]['value']:
                 mappings.append((pname, tgt_spec))
 
         if mappings:
