@@ -18,7 +18,9 @@ import uuid
 from contextlib import contextmanager
 from functools import lru_cache
 from pathlib import Path
-from typing import TYPE_CHECKING, Literal, TypedDict
+from typing import (
+    TYPE_CHECKING, ClassVar, Literal, TypedDict,
+)
 
 import bokeh.embed.wrappers
 import param
@@ -42,11 +44,20 @@ from .state import state
 if TYPE_CHECKING:
     from bokeh.resources import Urls
 
-    class ResourcesType(TypedDict):
+    class TarballType(TypedDict, total=False):
+        tar: str
+        src: str
+        dest: str
+        exclude: list[str]
+
+    class ResourcesType(TypedDict, total=False):
         css: dict[str, str]
-        js:  dict[str, str]
+        font: dict[str, str]
+        js: dict[str, str]
         js_modules: dict[str, str]
         raw_css: list[str]
+        tarball: dict[str, TarballType]
+        bundle: bool
 
 logger = logging.getLogger(__name__)
 
@@ -145,7 +156,7 @@ JS_URLS = {
     'bootstrap5': f'{CDN_DIST}bundled/bootstrap5/js/bootstrap.bundle.min.js'
 }
 
-extension_dirs['panel'] = str(DIST_DIR)
+extension_dirs['panel'] = DIST_DIR
 
 bokeh.embed.wrappers._ONLOAD = """\
 (function() {
@@ -222,11 +233,13 @@ def resolve_custom_path(
     path: pathlib.Path | None
     """
     if not path:
-        return
+        return None
     if not isinstance(obj, type):
         obj = type(obj)
     try:
         mod = importlib.import_module(obj.__module__)
+        if mod.__file__ is None:
+            return None
         module_path = Path(mod.__file__).parent
         assert module_path.exists()
     except Exception:
@@ -244,7 +257,7 @@ def resolve_custom_path(
     abs_path = abs_path.resolve()
     if not relative:
         return abs_path
-    return os.path.relpath(abs_path, module_path)
+    return pathlib.Path(os.path.relpath(abs_path, module_path))
 
 def component_resource_path(component, attr, path):
     """
@@ -469,7 +482,7 @@ class ResourceComponent:
     that have to be resolved.
     """
 
-    _resources = {
+    _resources: ClassVar[ResourcesType] = {
         'css': {},
         'font': {},
         'js': {},
@@ -478,7 +491,12 @@ class ResourceComponent:
     }
 
     @classmethod
-    def _resolve_resource(cls, resource_type: str, resource: str, cdn: bool = False):
+    def _resolve_resource(
+        cls,
+        resource_type: str,
+        resource: str,
+        cdn: bool = False
+    ) -> str:
         dist_path = get_dist_path(cdn=cdn)
         if resource.startswith(CDN_DIST):
             resource_path = resource.replace(f'{CDN_DIST}bundled/', '')
@@ -508,6 +526,9 @@ class ResourceComponent:
             return component_resource_path(
                 cls, f'_resources/{resource_type}', resource
             )
+        raise FileNotFoundError(
+            f'Could not resolve resource {resource!r}'
+        )
 
     def resolve_resources(
         self,
@@ -532,7 +553,7 @@ class ResourceComponent:
         Dictionary containing JS and CSS resources.
         """
         cls = type(self)
-        resources = {}
+        resources: ResourcesType = {}
         for rt, res in self._resources.items():
             if not isinstance(res, dict):
                 continue
@@ -543,9 +564,9 @@ class ResourceComponent:
                 for name, url in res.items()
             }
             if rt in resources:
-                resources[rt] = dict(resources[rt], **res)
+                resources[rt] = dict(resources[rt], **res)  # type: ignore
             else:
-                resources[rt] = res
+                resources[rt] = res  # type: ignore
 
         resource_types: ResourcesType = {
             'js': {},
@@ -556,20 +577,20 @@ class ResourceComponent:
 
         cdn = use_cdn() if cdn == 'auto' else cdn
         for resource_type in resource_types:
-            if resource_type not in resources or resource_type == 'raw_css':
+            if resource_type not in resources or resource_type == 'raw_css':  # type: ignore
                 continue
-            resource_files = resource_types[resource_type]
-            for rname, resource in resources[resource_type].items():
+            resource_files = resource_types[resource_type]  # type: ignore
+            for rname, resource in resources[resource_type].items():  # type: ignore
                 resolved_resource = self._resolve_resource(
                     resource_type, resource, cdn=cdn
                 )
                 if resolved_resource:
-                    resource_files[rname] = resolved_resource
+                    resource_files[rname] = resolved_resource  # type: ignore
 
         version_suffix = f'?v={JS_VERSION}'
         dist_path = get_dist_path(cdn=cdn)
         for resource_type, extra_resources in (extras or {}).items():
-            resource_files = resource_types[resource_type]
+            resource_files = resource_types[resource_type]  # type: ignore
             for name, res in extra_resources.items():
                 if not cdn:
                     res = res.replace(CDN_DIST, dist_path)
@@ -717,7 +738,7 @@ class Resources(BkResources):
     def css_files(self):
         from ..config import config
 
-        files = super(Resources, self).css_files
+        files = super().css_files
         self.extra_resources(files, '__css__')
         css_files = self.adjust_paths([
             css for css in files if self.mode != 'inline' or not is_cdn_url(css)
@@ -733,7 +754,7 @@ class Resources(BkResources):
     @property
     def css_raw(self):
         from ..config import config
-        raw = super(Resources, self).css_raw
+        raw = super().css_raw
 
         # Inline local dist resources
         css_files = self._collect_external_resources("__css__")
@@ -768,7 +789,7 @@ class Resources(BkResources):
 
         # Gather JS files
         with set_resource_mode(self.mode):
-            files = super(Resources, self).js_files
+            files = super().js_files
             self.extra_resources(files, '__javascript__')
         files += [js for js in config.js_files.values()]
         if config.design:
@@ -840,7 +861,7 @@ class Resources(BkResources):
 
     @property
     def js_raw(self):
-        raw_js = super(Resources, self).js_raw
+        raw_js = super().js_raw
         if not self.mode == 'inline':
             return raw_js
 
