@@ -6,9 +6,8 @@ from __future__ import annotations
 import io
 import os
 
-from typing import (
-    IO, TYPE_CHECKING, Any, Iterable, Optional,
-)
+from collections.abc import Iterable
+from typing import IO, TYPE_CHECKING, Any
 
 import bokeh
 
@@ -19,6 +18,7 @@ from bokeh.embed.util import (
 )
 from bokeh.io.export import get_screenshot_as_png
 from bokeh.model import Model
+from bokeh.models import UIElement
 from bokeh.resources import CDN, INLINE, Resources as BkResources
 from pyviz_comms import Comm
 
@@ -59,8 +59,12 @@ else
 bokeh.io.export._WAIT_SCRIPT = _WAIT_SCRIPT
 
 def save_png(
-    model: Model, filename: str, resources=CDN, template=None,
-    template_variables=None, timeout: int = 5
+    model: UIElement | Document,
+    filename: str | os.PathLike | IO,
+    resources: BkResources = CDN,
+    template=None,
+    template_variables: dict[str, Any] | None = None,
+    timeout: int = 5
 ) -> None:
     """
     Saves a bokeh model to png
@@ -104,7 +108,7 @@ def save_png(
         """
 
     try:
-        def get_layout_html(obj, resources, width, height, **kwargs):
+        def get_layout_html(obj: UIElement | Document, resources: Resources, width: int | None, height: int | None, **kwargs):
             resources = Resources.from_bokeh(resources)
             return file_html(
                 obj, resources, title="", template=template,
@@ -112,7 +116,7 @@ def save_png(
                 _always_new=True
             )
         old_layout_fn = bokeh.io.export.get_layout_html
-        bokeh.io.export.get_layout_html = get_layout_html
+        bokeh.io.export.get_layout_html = get_layout_html    # type: ignore
         img = get_screenshot_as_png(model, driver=webdriver, timeout=timeout, resources=resources)
 
         if img.width == 0 or img.height == 0:
@@ -140,8 +144,8 @@ def _title_from_models(models: Iterable[Model], title: str) -> str:
     return DEFAULT_TITLE
 
 def file_html(
-    models: Model | Document | list[Model], resources: Resources | None,
-    title: Optional[str] = None, template: Template | str = BASE_TEMPLATE,
+    models: Model | Document | list[Model], resources: BkResources | None,
+    title: str | None = None, template: Template | str = BASE_TEMPLATE,
     template_variables: dict[str, Any] = {}, theme: ThemeLike = None,
     _always_new: bool = False
 ):
@@ -159,7 +163,7 @@ def file_html(
         (docs_json, render_items) = standalone_docs_json_and_render_items(
             models_seq, suppress_callback_warning=True
         )
-        title = _title_from_models(models_seq, title)
+        title = _title_from_models(models_seq, title or 'Panel Application')
         bundle = bundle_resources(models_seq, resources)
         return html_page_for_render_items(
             bundle, docs_json, render_items, title=title, template=template,
@@ -171,11 +175,11 @@ def file_html(
 #---------------------------------------------------------------------
 
 def save(
-    panel: Viewable, filename: str | os.PathLike | IO, title: Optional[str]=None,
+    panel: Viewable | Document, filename: str | os.PathLike | IO, title: str | None = None,
     resources: BkResources | None = None, template: Template | str | None = None,
-    template_variables: dict[str, Any] = None, embed: bool = False,
+    template_variables: dict[str, Any] | None = None, embed: bool = False,
     max_states: int = 1000, max_opts: int = 3, embed_json: bool = False,
-    json_prefix: str = '', save_path: str = './', load_path: Optional[str] = None,
+    json_prefix: str = '', save_path: str = './', load_path: str | None = None,
     progress: bool = True, embed_states={}, as_png=None,
     **kwargs
 ) -> None:
@@ -252,7 +256,7 @@ def save(
     comm = Comm()
     with config.set(embed=embed):
         if isinstance(panel, Document):
-            model = panel
+            model: Document | Model = panel
         elif isinstance(panel, BaseTemplate):
             with set_resource_mode(mode):
                 panel._init_doc(doc, title=title)
@@ -268,6 +272,9 @@ def save(
             else:
                 add_to_doc(model, doc, True)
 
+    if isinstance(model, Model) and not isinstance(model, UIElement):
+        raise ValueError("Cannot render non-UI components.")
+
     if as_png:
         return save_png(
             model, resources=resources, filename=filename, template=template,
