@@ -33,6 +33,7 @@ from .base import (  # noqa
     Column, ListLike, ListPanel, Panel, Row, WidgetBox,
 )
 from .card import Card  # noqa
+from .feed import Feed  # noqa
 from .flex import FlexBox  # noqa
 from .float import FloatPanel  # noqa
 from .grid import GridBox, GridSpec  # noqa
@@ -48,6 +49,7 @@ __all__ = (
     "Card",
     "Column",
     "Divider",
+    "Feed",
     "FloatPanel",
     "FlexBox",
     "GridBox",
