@@ -1,8 +1,7 @@
 from __future__ import annotations
 
-from typing import (
-    TYPE_CHECKING, Callable, ClassVar, Mapping,
-)
+from collections.abc import Callable, Mapping
+from typing import TYPE_CHECKING, ClassVar
 
 import param
 
