@@ -1,16 +1,16 @@
 import {div} from "@bokehjs/core/dom"
 import type * as p from "@bokehjs/core/properties"
 import {isNumber} from "@bokehjs/core/util/types"
+import {LayoutDOM, LayoutDOMView} from "@bokehjs/models/layouts/layout_dom"
 import {ColumnDataSource} from "@bokehjs/models/sources/column_data_source"
 
 import {debounce} from  "debounce"
 
 import {transform_cds_to_records} from "./data"
-import {LayoutDOM, LayoutDOMView} from "@bokehjs/models/layouts/layout_dom"
+import {set_size} from "./layout"
+import {GL} from "./lumagl"
 import {makeTooltip} from "./tooltips"
 
-import GL from "@luma.gl/constants"
-
 function extractClasses() {
   // Get classes for registration from standalone deck.gl
   const classesDict: any = {}
@@ -19,21 +19,39 @@ function extractClasses() {
   for (const cls of classes) {
     classesDict[cls] = deck[cls]
   }
+  const carto = (window as any).CartoLibrary
+  const layers = Object.keys(carto.CARTO_LAYERS).filter(x => x.endsWith("Layer"))
+  for (const layer of layers) {
+    classesDict[layer] = carto.CARTO_LAYERS[layer]
+  }
   return classesDict
 }
 
+function extractFunctions() {
+  const carto = (window as any).CartoLibrary
+  const sources = Object.keys(carto.CARTO_SOURCES).filter(x => x.toLowerCase().endsWith("source"))
+  const functionDict: any = {}
+  for (const src of sources) {
+    functionDict[src] = carto.CARTO_SOURCES[src]
+  }
+  return functionDict
+}
+
 export class DeckGLPlotView extends LayoutDOMView {
   declare model: DeckGLPlot
 
   jsonConverter: any
   deckGL: any
   _connected: any[]
+  _map: any
   _layer_map: any
+  _view_cb: any
+  _initialized: boolean = false
 
   override connect_signals(): void {
     super.connect_signals()
-    const {data, mapbox_api_key, tooltip, layers, initialViewState, data_sources} = this.model.properties
-    this.on_change([mapbox_api_key, tooltip], () => this.render())
+    const {data, mapbox_api_key, tooltip, configuration, layers, initialViewState, data_sources} = this.model.properties
+    this.on_change([mapbox_api_key, tooltip, configuration], () => this.render())
     this.on_change([data, initialViewState], () => this.updateDeck())
     this.on_change([layers], () => this._update_layers())
     this.on_change([data_sources], () => this._connect_sources(true))
@@ -64,11 +82,14 @@ export class DeckGLPlotView extends LayoutDOMView {
 
   override initialize(): void {
     super.initialize()
+    const view_timeout = this.model.throttle.view || 200
+    this._view_cb = debounce((event: any) => this._on_viewState_event(event), view_timeout, false)
     if ((window as any).deck.JSONConverter) {
       const {CSVLoader, Tiles3DLoader} = (window as any).loaders;
       (window as any).loaders.registerLoaders([Tiles3DLoader, CSVLoader])
       const jsonConverterConfiguration: any = {
         classes: extractClasses(),
+        functions: extractFunctions(),
         // Will be resolved as `<enum-name>.<enum-value>`
         enumerations: {
           COORDINATE_SYSTEM: (window as any).deck.COORDINATE_SYSTEM,
@@ -151,21 +172,27 @@ export class DeckGLPlotView extends LayoutDOMView {
   }
 
   getData(): any {
-    const view_timeout = this.model.throttle.view || 200
     const hover_timeout = this.model.throttle.hover || 100
-    const view_cb = debounce((event: any) => this._on_viewState_event(event), view_timeout, false)
     const hover_cb = debounce((event: any) => this._on_hover_event(event), hover_timeout, false)
     const data = {
       ...this.model.data,
       layers: this.model.layers,
       initialViewState: this.model.initialViewState,
-      onViewStateChange: view_cb,
+      onViewStateChange: (event: any) => this._sync_viewstate(event),
       onClick: (event: any) => this._on_click_event(event),
       onHover: hover_cb,
     }
     return data
   }
 
+  _sync_viewstate(event: any): void {
+    if (this._map) {
+      const {longitude, latitude, ...rest} = event.viewState
+      this._map.jumpTo({center: [longitude, latitude], ...rest})
+    }
+    this._view_cb(event)
+  }
+
   updateDeck(): void {
     if (!this.deckGL) {
       this.render()
@@ -183,16 +210,36 @@ export class DeckGLPlotView extends LayoutDOMView {
   createDeck({mapboxApiKey, container, jsonInput, tooltip}: any): void {
     let deckgl
     try {
+      let configuration
+      if (this.model.configuration) {
+        configuration = eval(`(${  this.model.configuration  })`)
+      } else {
+        configuration = null
+      }
+      this.jsonConverter.mergeConfiguration(configuration)
       const props = this.jsonConverter.convert(jsonInput)
       const getTooltip = makeTooltip(tooltip, props.layers)
+      if (props.mapStyle === null) {
+        props.map = null
+      } else if (props.mapStyle.includes("carto")) {
+        this._map = new (window as any).maplibregl.Map({
+          container,
+          style: props.mapStyle,
+          interactive: false,
+        })
+        props.controller = true
+        props.map = null
+        delete props.mapStyle
+      } else {
+        props.map = (window as any).mapboxgl
+        props.mapboxApiAccessToken = mapboxApiKey
+      }
       deckgl = new (window as any).deck.DeckGL({
         ...props,
-        map: (window as any).mapboxgl,
-        mapboxApiAccessToken: mapboxApiKey,
         container,
         getTooltip,
-        width: "100%",
         height: "100%",
+        width: "100%",
       })
     } catch (err) {
       console.error(err)
@@ -203,11 +250,10 @@ export class DeckGLPlotView extends LayoutDOMView {
   override render(): void {
     super.render()
     const container = div({class: "deckgl"})
-
+    set_size(container, this.model, false)
     const MAPBOX_API_KEY = this.model.mapbox_api_key
     const tooltip = this.model.tooltip
     const data = this.getData()
-
     if ((window as any).deck.createDeck) {
       this.deckGL = (window as any).deck.createDeck({
         mapboxApiKey: MAPBOX_API_KEY,
@@ -224,11 +270,27 @@ export class DeckGLPlotView extends LayoutDOMView {
       })
     }
     this.shadow_el.appendChild(container)
+    this._initialized = false
+  }
+
+  resize(): void {
+    this.deckGL.redraw(true)
+    if (this._map) {
+      this._map.resize()
+    }
   }
 
   override after_layout(): void {
     super.after_layout()
-    this.deckGL.redraw(true)
+    if (!this._initialized) {
+      this.resize()
+    }
+    this._initialized = true
+  }
+
+  override after_resize(): void {
+    super.after_resize()
+    this.resize()
   }
 }
 
@@ -241,6 +303,7 @@ export namespace DeckGLPlot {
     layers: p.Property<any[]>
     mapbox_api_key: p.Property<string>
     tooltip: p.Property<any>
+    configuration: p.Property<string>
     clickState: p.Property<any>
     hoverState: p.Property<any>
     throttle: p.Property<any>
@@ -272,6 +335,7 @@ export class DeckGLPlot extends LayoutDOM {
       mapbox_api_key:   [ Str,                       "" ],
       throttle:         [ Any,                          {} ],
       tooltip:          [ Any,                        true ],
+      configuration:          [ Str,                        "" ],
       viewState:        [ Any,                          {} ],
     }))
 
