@@ -8,12 +8,12 @@ import functools
 import json
 import textwrap
 
-from typing import (
-    TYPE_CHECKING, Any, ClassVar, Mapping,
-)
+from collections.abc import Mapping
+from typing import TYPE_CHECKING, Any, ClassVar
 
 import param  # type: ignore
 
+from ..config import config
 from ..io.resources import CDN_DIST
 from ..models.markup import HTML as _BkHTML, JSON as _BkJSON, HTMLStreamEvent
 from ..util import HTML_SANITIZER, escape, prefix_length
@@ -36,7 +36,7 @@ class HTMLBasePane(ModelPane):
         Whether to enable streaming of text snippets. This is useful
         when updating a string step by step, e.g. in a chat message.""")
 
-    _bokeh_model: ClassVar[Model] = _BkHTML
+    _bokeh_model: ClassVar[type[Model]] = _BkHTML
 
     _rename: ClassVar[Mapping[str, str | None]] = {'object': 'text', 'enable_streaming': None}
 
@@ -168,7 +168,7 @@ class DataFrame(HTML):
     index_names = param.Boolean(default=True, doc="""
         Prints the names of the indexes.""")
 
-    justify = param.ObjectSelector(default=None, allow_None=True, objects=[
+    justify = param.Selector(default=None, allow_None=True, objects=[
         'left', 'right', 'center', 'justify', 'justify-all', 'start',
         'end', 'inherit', 'match-parent', 'initial', 'unset'], doc="""
         How to justify the column labels.""")
@@ -271,8 +271,8 @@ class DataFrame(HTML):
             if 'dask' in module:
                 html = obj.to_html(max_rows=self.max_rows).replace('border="1"', '')
             elif 'style' in module:
-                classes = ' '.join(classes)
-                html = obj.to_html(table_attributes=f'class="{classes}"')
+                class_string = ' '.join(classes)
+                html = obj.to_html(table_attributes=f'class="{class_string}"')
             else:
                 kwargs = {p: getattr(self, p) for p in self._rerender_params
                           if p not in HTMLBasePane.param and p not in ('_object', 'text_align')}
@@ -397,7 +397,7 @@ class Markdown(HTMLBasePane):
             return False
 
     @classmethod
-    @functools.lru_cache(maxsize=None)
+    @functools.cache
     def _get_parser(cls, renderer, plugins, **renderer_options):
         if renderer == 'markdown':
             return None
@@ -461,7 +461,7 @@ class Markdown(HTMLBasePane):
             html = markdown.markdown(
                 obj,
                 extensions=self.extensions,
-                output_format='html5',
+                output_format='xhtml',
                 **self.renderer_options
             )
         else:
@@ -482,8 +482,6 @@ class Markdown(HTMLBasePane):
             params['css_classes'] = ['markdown'] + params['css_classes']
         return super()._process_param_change(params)
 
-
-
 class JSON(HTMLBasePane):
     """
     The `JSON` pane allows rendering arbitrary JSON strings, dicts and other
@@ -505,14 +503,17 @@ class JSON(HTMLBasePane):
     hover_preview = param.Boolean(default=False, doc="""
         Whether to display a hover preview for collapsed nodes.""")
 
-    theme = param.ObjectSelector(default="dark", objects=["light", "dark"], doc="""
-        Whether the JSON tree view is expanded by default.""")
+    theme = param.Selector(default="light", objects=["light", "dark"], doc="""
+        If no value is provided, it defaults to the current theme
+        set by pn.config.theme, as specified in the
+        JSON.THEME_CONFIGURATION dictionary. If not defined there, it
+        falls back to the default parameter value.""")
 
     priority: ClassVar[float | bool | None] = None
 
     _applies_kw: ClassVar[bool] = True
 
-    _bokeh_model: ClassVar[Model] = _BkJSON
+    _bokeh_model: ClassVar[type[Model]] = _BkJSON
 
     _rename: ClassVar[Mapping[str, str | None]] = {
         "object": "text", "encoder": None, "style": "styles"
@@ -526,6 +527,13 @@ class JSON(HTMLBasePane):
         f'{CDN_DIST}css/json.css'
     ]
 
+    THEME_CONFIGURATION: ClassVar[dict[str,str]] = {"default": "light", "dark": "dark"}
+
+    def __init__(self, object=None, **params):
+        if "theme" not in params:
+            params["theme"]=self._get_theme(config.theme)
+        super().__init__(object=object, **params)
+
     @classmethod
     def applies(cls, obj: Any, **params) -> float | bool | None:
         if isinstance(obj, (list, dict)):
@@ -559,3 +567,7 @@ class JSON(HTMLBasePane):
         if 'depth' in params:
             params['depth'] = None if params['depth'] < 0 else params['depth']
         return params
+
+    @classmethod
+    def _get_theme(cls, config_theme: str)->str:
+        return cls.THEME_CONFIGURATION.get(config_theme, cls.param.theme.default)
