@@ -14,19 +14,17 @@ import textwrap
 import types
 
 from collections import defaultdict, namedtuple
-from collections.abc import Callable
+from collections.abc import Callable, Generator
 from contextlib import contextmanager
 from functools import partial
-from typing import (
-    TYPE_CHECKING, Any, ClassVar, Generator, Mapping, Optional,
-)
+from typing import TYPE_CHECKING, Any, ClassVar
 
 import param
 
 try:
     from param import Skip
 except Exception:
-    class Skip(Exception):
+    class Skip(Exception):  # type: ignore
         """
         Exception that allows skipping an update for function-level updates.
         """
@@ -36,14 +34,6 @@ from param.parameterized import (
 )
 from param.reactive import rx
 
-try:
-    from param import Skip
-except Exception:
-    class Skip(RuntimeError):
-        """
-        Exception that allows skipping an update for function-level updates.
-        """
-
 from .config import config
 from .io import state
 from .layout import (
@@ -85,13 +75,13 @@ def SingleFileSelector(pobj: param.Parameter) -> type[Widget]:
 
 def LiteralInputTyped(pobj: param.Parameter) -> type[Widget]:
     if isinstance(pobj, param.Tuple):
-        return type(str('TupleInput'), (LiteralInput,), {'type': tuple})
+        return type('TupleInput', (LiteralInput,), {'type': tuple})
     elif isinstance(pobj, param.Number):
-        return type(str('NumberInput'), (LiteralInput,), {'type': (int, float)})
+        return type('NumberInput', (LiteralInput,), {'type': (int, float)})
     elif isinstance(pobj, param.Dict):
-        return type(str('DictInput'), (LiteralInput,), {'type': dict})
+        return type('DictInput', (LiteralInput,), {'type': dict})
     elif isinstance(pobj, param.List):
-        return type(str('ListInput'), (LiteralInput,), {'type': list})
+        return type('ListInput', (LiteralInput,), {'type': list})
     return LiteralInput
 
 
@@ -214,7 +204,7 @@ class Param(Pane):
         Dictionary of widget overrides, mapping from parameter name
         to widget class.""")
 
-    mapping: ClassVar[Mapping[param.Parameter, Widget | Callable[[param.Parameter], Widget]]] = {
+    mapping: ClassVar[dict[param.Parameter, type[WidgetBase] | Callable[[param.Parameter], type[WidgetBase]]]] = {
         param.Action:            Button,
         param.Array:             ArrayInput,
         param.Boolean:           Checkbox,
@@ -723,10 +713,11 @@ class Param(Pane):
         return dict(widgets)
 
     def _get_model(
-        self, doc: Document, root: Optional[Model] = None,
-        parent: Optional[Model] = None, comm: Optional[Comm] = None
+        self, doc: Document, root: Model | None = None,
+        parent: Model | None = None, comm: Comm | None = None
     ) -> Model:
         model = self.layout._get_model(doc, root, parent, comm)
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
         return model
 
@@ -758,7 +749,7 @@ class Param(Pane):
             return wtype
 
     def get_root(
-        self, doc: Optional[Document] = None, comm: Comm | None = None,
+        self, doc: Document | None = None, comm: Comm | None = None,
         preprocess: bool = True
     ) -> Model:
         root = super().get_root(doc, comm, preprocess)
@@ -920,8 +911,8 @@ class ParamRef(ReplacementPane):
         self._replace_pane()
 
     def _get_model(
-        self, doc: Document, root: Optional[Model] = None,
-        parent: Optional[Model] = None, comm: Optional[Comm] = None
+        self, doc: Document, root: Model | None = None,
+        parent: Model | None = None, comm: Comm | None = None
     ) -> Model:
         if not self._evaled:
             deferred = self.defer_load and not state.loaded
@@ -1237,9 +1228,9 @@ class ReactiveExpr(Pane):
         return self.widget_layout(*widgets)
 
     def _get_model(
-        self, doc: Document, root: Optional['Model'] = None,
-        parent: Optional['Model'] = None, comm: Optional[Comm] = None
-    ) -> 'Model':
+        self, doc: Document, root: Model | None = None,
+        parent: Model | None = None, comm: Comm | None = None
+    ) -> Model:
         return self.layout._get_model(doc, root, parent, comm)
 
     def _generate_layout(self):
@@ -1312,7 +1303,7 @@ class JSONInit(param.Parameterized):
         if self.json_file or env_var.endswith('.json'):
             try:
                 fname = self.json_file if self.json_file else env_var
-                with open(fullpath(fname), 'r') as f:
+                with open(fullpath(fname)) as f:
                     spec = json.load(f)
             except Exception:
                 warnobj.warning(f'Could not load JSON file {spec!r}')
