@@ -68,7 +68,7 @@ def internet_available(host="8.8.8.8", port=53, timeout=3):
         with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as conn:
             conn.connect((host, port))
         return True
-    except socket.error:
+    except OSError:
         return False
 
 def port_open(port):
