@@ -4,7 +4,6 @@ import asyncio
 import unittest.mock
 
 from functools import partial
-from typing import ClassVar, Mapping
 
 import bokeh.core.properties as bp
 import param
@@ -67,7 +66,7 @@ def test_param_rename():
 
         a = param.Parameter()
 
-        _rename: ClassVar[Mapping[str, str | None]] = {'a': 'b'}
+        _rename = {'a': 'b'}
 
     obj = ReactiveRename()
 
