@@ -57,7 +57,7 @@ class Theme(param.Parameterized):
        A stylesheet that overrides variables specifically for the
        Theme subclass. In most cases, this is not necessary.""")
 
-    modifiers: ClassVar[dict[Viewable, dict[str, Any]]] = {}
+    modifiers: ClassVar[dict[type[Viewable], dict[str, Any]]] = {}
 
 
 BOKEH_DARK = dict(_dark_minimal.json)
@@ -97,10 +97,10 @@ class Design(param.Parameterized, ResourceComponent):
     theme = param.ClassSelector(class_=Theme, constant=True)
 
     # Defines parameter overrides to apply to each model
-    modifiers: ClassVar[dict[Viewable, dict[str, Any]]] = {}
+    modifiers: ClassVar[dict[type[Viewable], dict[str, Any]]] = {}
 
     # Defines the resources required to render this theme
-    _resources: ClassVar[dict[str, dict[str, str]]] = {}
+    _resources = {}
 
     # Declares valid themes for this Design
     _themes: ClassVar[dict[str, type[Theme]]] = {
@@ -108,7 +108,7 @@ class Design(param.Parameterized, ResourceComponent):
         'dark': DarkTheme
     }
 
-    _cache = {}
+    _cache: ClassVar[dict[str, ImportedStyleSheet]] = {}
 
     def __init__(self, theme=None, **params):
         if isinstance(theme, type) and issubclass(theme, Theme):
@@ -119,8 +119,8 @@ class Design(param.Parameterized, ResourceComponent):
         super().__init__(theme=theme, **params)
 
     def _reapply(
-        self, viewable: Viewable, root: Model, old_models: list[Model] = None,
-        isolated: bool=True, cache=None, document=None
+        self, viewable: Viewable, root: Model, old_models: list[Model] | None = None,
+        isolated: bool = True, cache=None, document: Document | None = None
     ) -> None:
         ref = root.ref['id']
         seen = set()
@@ -139,12 +139,17 @@ class Design(param.Parameterized, ResourceComponent):
 
     def _apply_hooks(self, viewable: Viewable, root: Model, changed: Viewable, old_models=None) -> None:
         from ..io.state import state
-        if root.document in state._stylesheets:
+        if root.document is None:
+            cache: dict[str, ImportedStyleSheet] = {}
+        elif root.document in state._stylesheets:
             cache = state._stylesheets[root.document]
         else:
             state._stylesheets[root.document] = cache = {}
-        with root.document.models.freeze():
-            self._reapply(changed, root, old_models, isolated=False, cache=cache, document=root.document)
+        if root.document:
+            with root.document.models.freeze():
+                self._reapply(changed, root, old_models, isolated=False, cache=cache, document=root.document)
+        else:
+            self._reapply(changed, root, old_models, isolated=False, cache=cache)
 
     def _wrapper(self, viewable):
         return viewable
@@ -186,14 +191,14 @@ class Design(param.Parameterized, ResourceComponent):
 
     @classmethod
     def _get_modifiers(
-        cls, viewable: Viewable, theme: Theme = None, isolated: bool = True
+        cls, viewable: Viewable, theme: Theme | None = None, isolated: bool = True
     ):
         from ..io.resources import (
             CDN_DIST, component_resource_path, resolve_custom_path,
         )
         theme_type = type(theme) if isinstance(theme, Theme) else theme
         is_server = bool(state.curdoc.session_context) if not state._is_pyodide and state.curdoc else False
-        modifiers, child_modifiers = cls._resolve_modifiers(type(viewable), theme_type, is_server=is_server)
+        modifiers, child_modifiers = cls._resolve_modifiers(type(viewable), theme_type, is_server=is_server)  # type: ignore
         modifiers = dict(modifiers)
         if 'stylesheets' in modifiers:
             if isolated:
@@ -215,7 +220,7 @@ class Design(param.Parameterized, ResourceComponent):
         return modifiers, child_modifiers
 
     @classmethod
-    def _patch_modifiers(cls, doc, modifiers, cache):
+    def _patch_modifiers(cls, doc: Document | None, modifiers: dict[str, Any], cache: dict[str, ImportedStyleSheet]):
         if 'stylesheets' in modifiers:
             stylesheets = []
             for sts in modifiers['stylesheets']:
@@ -306,7 +311,7 @@ class Design(param.Parameterized, ResourceComponent):
     # Public API
     #----------------------------------------------------------------
 
-    def apply(self, viewable: Viewable, root: Model, isolated: bool=True):
+    def apply(self, viewable: Viewable, root: Model, isolated: bool = True):
         """
         Applies the Design to a Viewable and all it children.
 
