@@ -12,6 +12,7 @@ from __future__ import annotations
 
 import asyncio
 import functools
+import inspect
 import logging
 import os
 import sys
@@ -20,8 +21,9 @@ import traceback
 import typing
 import uuid
 
+from collections.abc import Callable, Mapping
 from typing import (
-    IO, TYPE_CHECKING, Any, Callable, ClassVar, Mapping, Optional,
+    IO, TYPE_CHECKING, Any, ClassVar,
 )
 
 import param  # type: ignore
@@ -54,13 +56,15 @@ if TYPE_CHECKING:
     from bokeh.model import Model
     from bokeh.server.contexts import BokehSessionContext
     from bokeh.server.server import Server
+    from typing_extensions import Self
 
     from .io.location import Location
+    from .io.notebook import Mimebundle
     from .io.server import StoppableThread
     from .theme import Design
 
 
-_tasks = set()
+_tasks: set[asyncio.Task] = set()
 
 
 class Layoutable(param.Parameterized):
@@ -69,7 +73,7 @@ class Layoutable(param.Parameterized):
     for all Panel components with a visual representation.
     """
 
-    align = Align(default='start', doc="""
+    align = Align(default="start", doc="""
         Whether the object should be aligned with the start, end or
         center of its container. If set as a tuple it will declare
         (vertical, horizontal) alignment.""")
@@ -86,7 +90,7 @@ class Layoutable(param.Parameterized):
     css_classes = param.List(default=[], nested_refs=True, doc="""
         CSS classes to apply to the layout.""")
 
-    design = param.ObjectSelector(default=None, objects=[], doc="""
+    design = param.Selector(default=None, objects=[], doc="""
         The design system to use to style components.""")
 
     height = param.Integer(default=None, bounds=(0, None), doc="""
@@ -127,7 +131,7 @@ class Layoutable(param.Parameterized):
         The width of the component (in pixels). This can be either
         fixed or preferred width, depending on width sizing policy.""")
 
-    width_policy = param.ObjectSelector(
+    width_policy = param.Selector(
         default="auto", objects=['auto', 'fixed', 'fit', 'min', 'max'], doc="""
         Describes how the component should maintain its width.
 
@@ -159,7 +163,7 @@ class Layoutable(param.Parameterized):
             management and other factors.
     """)
 
-    height_policy = param.ObjectSelector(
+    height_policy = param.Selector(
         default="auto", objects=['auto', 'fixed', 'fit', 'min', 'max'], doc="""
         Describes how the component should maintain its height.
 
@@ -191,7 +195,7 @@ class Layoutable(param.Parameterized):
             management and other factors.
     """)
 
-    sizing_mode = param.ObjectSelector(default=None, objects=[
+    sizing_mode = param.Selector(default=None, objects=[
         'fixed', 'stretch_width', 'stretch_height', 'stretch_both',
         'scale_width', 'scale_height', 'scale_both', None], doc="""
         How the component should size itself.
@@ -308,7 +312,11 @@ class ServableMixin:
     """
 
     def _modify_doc(
-        self, server_id: str, title: str, doc: Document, location: Optional['Location']
+        self,
+        server_id: str | None,
+        title: str,
+        doc: Document,
+        location: Location | bool | None
     ) -> Document:
         """
         Callback to handle FunctionHandler document creation.
@@ -318,10 +326,13 @@ class ServableMixin:
         return self.server_doc(doc, title, location) # type: ignore
 
     def _add_location(
-        self, doc: Document, location: Optional['Location' | bool],
-        root: Optional['Model'] = None
-    ) -> 'Location':
+        self,
+        doc: Document,
+        location: Location | bool,
+        root: Model | None = None
+    ) -> Location | None:
         from .io.location import Location
+        loc: Location | None
         if isinstance(location, Location):
             loc = location
             state._locations[doc] = loc
@@ -330,6 +341,9 @@ class ServableMixin:
         else:
             with set_curdoc(doc):
                 loc = state.location
+        if loc is None:
+            return None
+
         if root is None:
             loc_model = loc.get_root(doc)
         else:
@@ -344,9 +358,9 @@ class ServableMixin:
     #----------------------------------------------------------------
 
     def servable(
-        self, title: Optional[str] = None, location: bool | 'Location' = True,
-        area: str = 'main', target: Optional[str] = None
-    ) -> 'ServableMixin':
+        self, title: str | None = None, location: bool | Location = True,
+        area: str = 'main', target: str | None = None
+    ) -> Self:
         """
         Serves the object or adds it to the configured
         pn.state.template if in a `panel serve` context, writes to the
@@ -373,7 +387,8 @@ class ServableMixin:
         -------
         The Panel object itself
         """
-        if curdoc_locked().session_context:
+        doc = curdoc_locked()
+        if doc and doc.session_context:
             logger = logging.getLogger('bokeh')
             for handler in logger.handlers:
                 if isinstance(handler, logging.StreamHandler):
@@ -414,10 +429,10 @@ class ServableMixin:
         return self
 
     def show(
-        self, title: Optional[str] = None, port: int = 0, address: Optional[str] = None,
-        websocket_origin: Optional[str] = None, threaded: bool = False, verbose: bool = True,
-        open: bool = True, location: bool | 'Location' = True, **kwargs
-    ) -> 'StoppableThread' | 'Server':
+        self, title: str | None = None, port: int = 0, address: str | None = None,
+        websocket_origin: str | None = None, threaded: bool = False, verbose: bool = True,
+        open: bool = True, location: bool | Location = True, **kwargs
+    ) -> StoppableThread | Server:
         """
         Starts a Bokeh server and displays the Viewable in a new tab.
 
@@ -548,9 +563,9 @@ class Renderable(param.Parameterized, MimeRenderMixin):
         getattr(self._logger, level)(f'Session %s {msg}', id(state.curdoc), *args)
 
     def _get_model(
-        self, doc: Document, root: Optional['Model'] = None,
-        parent: Optional['Model'] = None, comm: Optional[Comm] = None
-    ) -> 'Model':
+        self, doc: Document, root: Model | None = None,
+        parent: Model | None = None, comm: Comm | None = None
+    ) -> Model:
         """
         Converts the objects being wrapped by the viewable into a
         bokeh model that can be composed in a bokeh layout.
@@ -587,7 +602,7 @@ class Renderable(param.Parameterized, MimeRenderMixin):
         if ref in state._handles:
             del state._handles[ref]
 
-    def _preprocess(self, root: 'Model', changed=None, old_models=None) -> None:
+    def _preprocess(self, root: Model, changed=None, old_models=None) -> None:
         """
         Applies preprocessing hooks to the root model.
 
@@ -600,12 +615,12 @@ class Renderable(param.Parameterized, MimeRenderMixin):
         changed = self if changed is None else changed
         hooks = self._preprocessing_hooks+self._hooks
         for hook in hooks:
-            try:
+            if len(inspect.signature(hook).parameters) >= 4:
                 hook(self, root, changed, old_models)
-            except TypeError:
+            else:
                 hook(self, root)
 
-    def _render_model(self, doc: Optional[Document] = None, comm: Optional[Comm] = None) -> 'Model':
+    def _render_model(self, doc: Document | None = None, comm: Comm | None = None) -> Model:
         if doc is None:
             doc = Document()
         if comm is None:
@@ -626,7 +641,7 @@ class Renderable(param.Parameterized, MimeRenderMixin):
     def _init_params(self) -> Mapping[str, Any]:
         return {k: v for k, v in self.param.values().items() if v is not None}
 
-    def _server_destroy(self, session_context: 'BokehSessionContext') -> None:
+    def _server_destroy(self, session_context: BokehSessionContext) -> None:
         """
         Server lifecycle hook triggered when session is destroyed.
         """
@@ -645,7 +660,7 @@ class Renderable(param.Parameterized, MimeRenderMixin):
                                         params=', '.join(param_reprs(self)))
 
     def get_root(
-        self, doc: Optional[Document] = None, comm: Optional[Comm] = None,
+        self, doc: Document | None = None, comm: Comm | None = None,
         preprocess: bool = True
     ) -> Model:
         """
@@ -683,6 +698,7 @@ class Renderable(param.Parameterized, MimeRenderMixin):
         state._views[ref] = (root_view, root, doc, comm)
         return root
 
+
 class Viewable(Renderable, Layoutable, ServableMixin):
     """
     Viewable is the baseclass all visual components in the panel
@@ -697,7 +713,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         Whether or not the Viewable is loading. If True a loading spinner
         is shown on top of the Viewable.""")
 
-    _preprocessing_hooks: ClassVar[list[Callable[['Viewable', 'Model'], None]]] = []
+    _preprocessing_hooks: ClassVar[list[Callable[[Viewable, Model], None]]] = []
 
     def __init__(self, **params):
         hooks = params.pop('hooks', [])
@@ -732,7 +748,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         else:
             stop_loading_spinner(self)
 
-    def _render_model(self, doc: Optional[Document] = None, comm: Optional[Comm] = None) -> 'Model':
+    def _render_model(self, doc: Document | None = None, comm: Comm | None = None) -> Model:
         if doc is None:
             doc = Document()
         if comm is None:
@@ -841,7 +857,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
     # Public API
     #----------------------------------------------------------------
 
-    def clone(self, **params) -> 'Viewable':
+    def clone(self, **params) -> Viewable:
         """
         Makes a copy of the object sharing the same parameters.
 
@@ -857,8 +873,8 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         return type(self)(**dict(inherited, **params))
 
     def select(
-        self, selector: Optional[type | Callable[['Viewable'], bool]] = None
-    ) -> list['Viewable']:
+        self, selector: type | Callable[[Viewable], bool] | None = None
+    ) -> list[Viewable]:
         """
         Iterates over the Viewable and any potential children in the
         applying the Selector.
@@ -882,9 +898,9 @@ class Viewable(Renderable, Layoutable, ServableMixin):
 
     def embed(
         self, max_states: int = 1000, max_opts: int = 3, json: bool = False,
-        json_prefix: str = '', save_path: str = './', load_path: Optional[str] = None,
+        json_prefix: str = '', save_path: str = './', load_path: str | None = None,
         progress: bool = False, states={}
-    ) -> None:
+    ) -> Mimebundle:
         """
         Renders a static version of a panel in a notebook by evaluating
         the set of states defined by the widgets in the model. Note
@@ -916,11 +932,11 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         )
 
     def save(
-        self, filename: str | os.PathLike | IO, title: Optional[str] = None,
+        self, filename: str | os.PathLike | IO, title: str | None = None,
         resources: Resources | None = None, template: str | Template | None = None,
         template_variables: dict[str, Any] = {}, embed: bool = False,
         max_states: int = 1000, max_opts: int = 3, embed_json: bool = False,
-        json_prefix: str='', save_path: str='./', load_path: Optional[str] = None,
+        json_prefix: str='', save_path: str='./', load_path: str | None = None,
         progress: bool = True, embed_states: dict[Any, Any] = {},
         as_png: bool | None = None, **kwargs
     ) -> None:
@@ -969,8 +985,8 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         )
 
     def server_doc(
-        self, doc: Optional[Document] = None, title: Optional[str] = None,
-        location: bool | 'Location' = True
+        self, doc: Document | None = None, title: str | None = None,
+        location: bool | Location = True
     ) -> Document:
         """
         Returns a serveable bokeh Document with the panel attached
@@ -1016,13 +1032,17 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         if location:
             self._add_location(doc, location, model)
         if config.notifications and doc is state.curdoc:
-            notification_model = state.notifications.get_root(doc)
-            notification_model.name = 'notifications'
-            doc.add_root(notification_model)
+            notification = state.notifications
+            if notification:
+                notification_model = notification.get_root(doc)
+                notification_model.name = 'notifications'
+                doc.add_root(notification_model)
         if config.browser_info and doc is state.curdoc:
-            browser_model = state.browser_info._get_model(doc, model)
-            browser_model.name = 'browser_info'
-            doc.add_root(browser_model)
+            browser = state.browser_info
+            if browser:
+                browser_model = browser._get_model(doc, model)
+                browser_model.name = 'browser_info'
+                doc.add_root(browser_model)
         return doc
 
 
@@ -1052,18 +1072,18 @@ class Viewer(param.Parameterized):
         return view
 
     def servable(
-        self, title: Optional[str]=None, location: bool | 'Location' = True,
-        area: str = 'main', target: Optional[str] = None
+        self, title: str | None=None, location: bool | Location = True,
+        area: str = 'main', target: str | None = None
     ) -> Viewable:
         return self._create_view().servable(title, location, area, target)
 
     servable.__doc__ = ServableMixin.servable.__doc__
 
     def show(
-        self, title: Optional[str] = None, port: int = 0, address: Optional[str] = None,
-        websocket_origin: Optional[str] = None, threaded: bool = False, verbose: bool = True,
-        open: bool = True, location: bool | 'Location' = True, **kwargs
-    ) -> threading.Thread | 'Server':
+        self, title: str | None = None, port: int = 0, address: str | None = None,
+        websocket_origin: str | None = None, threaded: bool = False, verbose: bool = True,
+        open: bool = True, location: bool | Location = True, **kwargs
+    ) -> threading.Thread | Server:
         return self._create_view().show(
             title, port, address, websocket_origin, threaded,
             verbose, open, location, **kwargs
@@ -1083,7 +1103,7 @@ class Child(param.ClassSelector):
     by calling the `pn.panel` utility.
     """
 
-    @typing.overload
+    @typing.overload  # type: ignore
     def __init__(
         self,
         default=None, *, is_instance=True, allow_None=False, doc=None,
